@@ -6,7 +6,7 @@ import importlib
 import JinjaStaticRenderer as renderer
 import Reader as Reader
 import Core as Core
-
+import Extensions as Extensions
 
 class Wrangler():
 
@@ -108,24 +108,55 @@ class Wrangler():
         self._reader = Reader.Reader(self.config)
         self._writer = Core.Writer(self.config["output_dir"], self.config["output_file_extension"], self._reporter)
         self._renderer = renderer.JinjaStaticRenderer(self.config, self._reporter, self._writer)
-        
-        self.items = self._reader.fetch();
-        
-        for item in self.items:
-            item.set_output_path(self._writer.generate_output_path(item.relpath()));
+
+        self.root_node = self._reader.fetch()
+
+        # Set output paths before trying to render anything.
+        # Two loops not so efficient... hmmm
+        def set_output(node):
+            if node.tag == 'file':
+                cargo = node.get_cargo()
+                if cargo:
+                    cargo.set_output_path(self._writer.generate_output_path(cargo.relpath()))
+            else:
+                for child in node.children:
+                    set_output(child)
+
+        set_output(self.root_node)
+
 
         if "WranglerHooks" in sys.modules and hasattr(sys.modules["WranglerHooks"], "BeforeRender"):
-            hook = sys.modules["WranglerHooks"].BeforeRender(self.config, self._reader.get_files(), self._renderer)
-            hook.process(self.items)
+            hook = sys.modules["WranglerHooks"].BeforeRender(self.config, self._renderer)
+            hook.process(self.root_node)
+
+        self.config["site_vars"]["sitemap"] = Extensions.SiteMap().build(self.root_node)
+
+
+
+        # Recursive render
+        def render_item(node):
+            if node.tag == 'file':
+                cargo = node.get_cargo()
+                if cargo:
+                    cargo.set_parents(node.get_parents())
+                    cargo.set_siblings(node.get_siblings())
+                    cargo.set_unique_siblings(node.get_siblings())
+                    cargo.set_children(node.get_child_pages())
+                    cargo.set_parents_siblings(node.get_parents_siblings())
+                    self._writer.save(self._renderer.render(cargo))
+            else:
+                for child in node.children:
+                    render_item(child)
+
+        render_item(self.root_node)
+
+
 
-        # print self.config["site_vars"]
 
-        for item in self.items:
-            self._writer.save(self._renderer.render(item))
 
         if "WranglerHooks" in sys.modules and hasattr(sys.modules["WranglerHooks"], "AfterRender"):
-            hook = sys.modules["WranglerHooks"].AfterRender(self.config, self._reader.get_files(), self._renderer)
-            hook.process(self.items)
+            hook = sys.modules["WranglerHooks"].AfterRender(self.config, self._renderer)
+            hook.process(self.root_node)
             
         self._reporter.set_last_build_time()
 
